/*
 *
 *  *  Copyright 2010-2017 OrientDB LTD (http://orientdb.com)
 *  *
 *  *  Licensed under the Apache License, Version 2.0 (the "License");
 *  *  you may not use this file except in compliance with the License.
 *  *  You may obtain a copy of the License at
 *  *
 *  *       http://www.apache.org/licenses/LICENSE-2.0
 *  *
 *  *  Unless required by applicable law or agreed to in writing, software
 *  *  distributed under the License is distributed on an "AS IS" BASIS,
 *  *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  *  See the License for the specific language governing permissions and
 *  *  limitations under the License.
 *  *
 *  * For more information: http://orientdb.com
 *
 */

package com.orientechnologies.orient.core.storage.impl.local;

/**
 * Listener which is called when it is detected that data page in file is broken. Typically it means
 * that magic number stored inside of the page of hash code generated by data does not match values
 * stored in page itself. Listener should either delegate code to other thread or should execute
 * operations which will consume very small amount of time.
 */
public interface OPageIsBrokenListener {
  /**
   * Method is called by write cache when it is detected that data page in file is broken.
   *
   * @param fileName Name of broken file.
   * @param pageIndex Index of page which is broken in file.
   */
  void pageIsBroken(String fileName, long pageIndex);
}
